using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace MeteorHunt
{
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        SpriteFont gameFont, gameFontLarge;

        //ljud
        AudioEngine audioEngine;
        WaveBank waveBank;
        SoundBank soundBank;

        //onda object p skrmen
        public static List<GameObj> evilObjList = new List<GameObj>();
        
        //meteoriter
        int respawnTime;
        int respawnRate = 1500;

        //goda objekt p skrmen
        public static List<GameObj> goodObjList = new List<GameObj>();

        //neutrala objekt p skrmen
        public static List<GameObj> neutralObjList = new List<GameObj>();

        Player player;

        //statiska medlemmar
        public static Random myRandom;
        public static int score;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            IsFixedTimeStep = false;
            Content.RootDirectory = "Content";
        }

        protected override void Initialize()
        {
            audioEngine = new AudioEngine("Content\\Audio\\GameAudio.xgs");
            waveBank = new WaveBank(audioEngine, "Content\\Audio\\Wave Bank.xwb");
            soundBank = new SoundBank(audioEngine, "Content\\Audio\\Sound Bank.xsb");

            base.Initialize();

            myRandom = new Random();
        }

        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            gameFont = Content.Load<SpriteFont>("gameFont");
            gameFontLarge = Content.Load<SpriteFont>("gameFontLarge");

            player = new Player(Content, soundBank);
        }

        protected override void UnloadContent()
        {
            soundBank.Dispose();
            waveBank.Dispose();
            audioEngine.Dispose();
        }

        private void Restart()
        {
            neutralObjList.Clear();
            evilObjList.Clear();
            goodObjList.Clear();
            score = 0;

            player = new Player(Content, soundBank);
        }

        protected override void Update(GameTime gameTime)
        {
            audioEngine.Update();

            if (!player.alive)
            {
                KeyboardState ks = Keyboard.GetState();
                if (ks.IsKeyDown(Keys.Enter))
                    Restart();
                return;
            }
            respawnTime += gameTime.ElapsedGameTime.Milliseconds;

            //uppdatera player
            player.Update(gameTime);

            //dags fr en ny meteorit
            if (respawnTime > respawnRate)
            {
                respawnTime = 0;
                evilObjList.Add(new Meteor(Content, soundBank));
            }
            
            //uppdatera neutrala objekt
            for(int i=0; i<neutralObjList.Count; i++)
            {
                neutralObjList[i].Update(gameTime);
                if (!neutralObjList[i].alive)
                {
                    neutralObjList.RemoveAt(i);
                    continue;
                }
            }

            //uppdatera goda objekt
            foreach (GameObj g in goodObjList)
            {
                g.Update(gameTime);
            }

            //uppdatera & rensa onda objekt
            for (int i = 0; i < evilObjList.Count; i++)
            {
                //uppdatera onda objekt
                evilObjList[i].Update(gameTime);

                //testa mot player
                evilObjList[i].CheckCollision(player);

                //g igenom alla goda fr att testa kollision
                for (int j = 0; j < goodObjList.Count; j++)
                {
                    //testa kollision
                    evilObjList[i].CheckCollision(goodObjList[j]);

                    //rensa bort dda objekt
                    if (!goodObjList[j].alive)
                    {
                        goodObjList.RemoveAt(j);
                        continue;
                    }
                }

                //rensa bort dda objekt
                if (!evilObjList[i].alive)
                {
                    evilObjList.RemoveAt(i);
                    continue;
                }
            }

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.Black);

            spriteBatch.Begin();

            //rita ut player
            player.Draw(spriteBatch);

            //rita ut onda objekt
            foreach (GameObj g in evilObjList)
            {
                g.Draw(spriteBatch);
            }

            //rita ut goda objekt
            foreach (GameObj g in goodObjList)
            {
                g.Draw(spriteBatch);
            }

            //rita ut neutrala objekt
            foreach (GameObj g in neutralObjList)
            {
                g.Draw(spriteBatch);
            }

            spriteBatch.DrawString(gameFont, "Score: " + score, new Vector2(10, 10), Color.White);

            if (!player.alive)
            {
                spriteBatch.DrawString(gameFontLarge, "G A M E   O V E R !", new Vector2(200, 300), Color.White);
                spriteBatch.DrawString(gameFontLarge, "press enter to restart", new Vector2(180, 350), Color.White);
            }

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
