using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;


namespace MeteorHunt
{
    public abstract class GameObj
    {
        public bool alive;
        public int life;
        public int damage;
        protected Vector2 position;
        protected Vector2 speed;
        protected float angle;
        protected float radius;
        protected float scale;
        protected Texture2D gfx;
        protected ContentManager content;
        protected SoundBank soundBank;


        public GameObj(ContentManager content, SoundBank soundBank)
        {
            this.soundBank = soundBank;
            this.content = content;
            this.scale = 1.0f;
            this.alive = true;
            this.position = new Vector2();
            this.speed = new Vector2();
        }

        public virtual void Update(GameTime gameTime)
        {
            position += speed;
        }

        public virtual void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(gfx, position, null, Color.White, 
                angle, new Vector2(gfx.Width / 2, gfx.Height / 2), 
                scale, SpriteEffects.None, 0);
        }

        public abstract void LastWishes();

        public void CheckCollision(GameObj g)
        {
            //inom radien? dvs kollision?

            float xdiff = this.position.X - g.position.X;
            float ydiff = this.position.Y - g.position.Y;

            if ((xdiff * xdiff + ydiff * ydiff) < (this.radius + g.radius) * (this.radius + g.radius))
            {
                //bda fremlen ska vara levande vid kollision
                if (g.alive && this.alive)
                {
                    g.life -= this.damage;
                    this.life -= g.damage;

                    if (this.life < 0)
                    {
                        this.alive = false;
                        this.LastWishes();
                    }
                    if (g.life <= 0)
                    {
                        g.alive = false;
                        g.LastWishes();
                    }
                }
            }
        }
    }
}
